from mpscenes.goals.static_sub_goal import StaticSubGoal
from mpscenes.goals.dynamic_sub_goal import DynamicSubGoal

staticGoalDict = {
    "weight": 1.0,
    "is_primary_goal": True,
    "indices": [0, 1],
    "parent_link": 0,
    "child_link": 3,
    "desired_position": [1, 2],
    "epsilon": 0.2,
    "type": "staticSubGoal",
}

staticGoal = StaticSubGoal(name="goal1", content_dict=staticGoalDict)
lineGoalDict = {
    "weight": 1.0,
    "is_primary_goal": False,
    "indices": [1],
    "parent_link": 0,
    "child_link": 3,
    "desired_position": [0.5],
    "angle": 0.5,
    "epsilon": 0.2,
    "type": "staticSubGoal",
}

lineGoal = StaticSubGoal(name="goal2", content_dict=lineGoalDict)
analyticGoalDict = {
    "weight": 1.0,
    "is_primary_goal": True,
    "indices": [0, 1],
    "parent_link": 0,
    "child_link": 3,
    "trajectory": ["1 + sp.cos(0.3 * t) + 1", "2.0"],
    "epsilon": 0.2,
    "type": "analyticSubGoal",
}
analyticGoal = DynamicSubGoal(name="goal2", content_dict=analyticGoalDict)
splineDict = {
    "degree": 2,
    "controlPoints": [[-2.0, 1.0], [2.0, 0.0], [4.0, 2.0], [3.0, 2.0]],
    "duration": 10,
}
splineGoalDict = {
    "weight": 1.0,
    "is_primary_goal": True,
    "indices": [0, 1],
    "parent_link": 0,
    "child_link": 3,
    "trajectory": splineDict,
    "epsilon": 0.2,
    "type": "splineSubGoal",
}
splineGoal = DynamicSubGoal(name="goal2", content_dict=splineGoalDict)



import numpy as np
import time
from abc import abstractmethod
from gym import spaces
import logging

from planarenvs.planar_common.planar_env import PlanarEnv


class PointRobotEnv(PlanarEnv):

    MAX_VEL = 10
    MAX_POS = 10
    MAX_ACC = 10
    MAX_FOR = 100

    def __init__(self, n=2, dt=0.01, render=False):
        super().__init__(render=render, dt=dt)
        self.n = n
        self._limits = {
            "pos": {
                "high": np.ones(self._n) * self.MAX_POS,
                "low": np.ones(self._n) * -self.MAX_POS,
            },
            "vel": {
                "high": np.ones(self._n) * self.MAX_VEL,
                "low": np.ones(self._n) * -self.MAX_VEL,
            },
            "acc": {
                "high": np.ones(self._n) * self.MAX_ACC,
                "low": np.ones(self._n) * -self.MAX_ACC,
            },
            "for": {
                "high": np.ones(self._n) * self.MAX_FOR,
                "low": np.ones(self._n) * -self.MAX_FOR,
            },
        }
        self._lim_up_pos = self._limits["pos"]["high"]
        self._lim_up_vel = self._limits["vel"]["high"]
        self._lim_up_acc = self._limits["acc"]["high"]
        self._lim_up_for = self._limits["for"]["high"]
        self.set_spaces()

    def reset_limits(self, **kwargs):
        for key in kwargs.keys() & self._limits.keys():
            limit_candidate = kwargs.get(key)
            for limit in ["low", "high"] & limit_candidate.keys():
                if limit_candidate[limit].size == self._n:
                    self._limits[key][limit] = limit_candidate[limit]
                else:
                    logging.warning(
                        "%s", "Incorrect size of limit." + "Resizing ignored."
                    )

        self._lim_up_pos = self._limits["pos"]["high"]
        self._lim_up_vel = self._limits["vel"]["high"]
        self._lim_up_acc = self._limits["acc"]["high"]
        self._lim_up_for = self._limits["for"]["high"]
        self.observation_space.spaces["joint_state"]["position"] = spaces.Box(
            low=-self._lim_up_pos, high=self._lim_up_pos, dtype=np.float64
        )
        self.observation_space.spaces["xdot"] = spaces.Box(
            low=-self._lim_up_vel, high=self._lim_up_vel, dtype=np.float64
        )

    @abstractmethod
    def set_spaces(self):
        pass

    def _reward(self):
        reward = -1.0 if not self._terminal() else 0.0
        return reward

    def _terminal(self):
        return False

    @abstractmethod
    def continuous_dynamics(self, x, t):
        pass

    def render(self, mode="human"):
        self.render_common(self._limits)
        from gym.envs.classic_control import (  # pylint: disable=import-outside-toplevel
            rendering,
        )

        # drawAxis
        self._viewer.draw_line(
            (self._limits["pos"]["low"][0], 0),
            (self._limits["pos"]["high"][0], 0),
        )
        self._viewer.draw_line(
            (0, self._limits["pos"]["low"][1]),
            (0, self._limits["pos"]["high"][1]),
        )
        # drawPoint
        x = self._state["joint_state"]["position"][0:2]
        tf0 = rendering.Transform(rotation=0, translation=(x[0], x[1]))
        joint = self._viewer.draw_circle(0.10)
        joint.set_color(0.8, 0.8, 0)
        joint.add_attr(tf0)
        time.sleep(self.dt())

        return self._viewer.render(return_rgb_array=mode == "rgb_array")



import numpy as np
from gym import spaces


class AccEnv(object):
    def set_spaces(self):
        self.observation_space = spaces.Dict(
            {"joint_state": spaces.Dict({
                "position": spaces.Box(
                    low=-self._lim_up_pos, high=self._lim_up_pos, dtype=np.float64
                ),
                "velocity": spaces.Box(
                    low=-self._lim_up_vel, high=self._lim_up_vel, dtype=np.float64
                ),
            })}
        )
        self.action_space = spaces.Box(
            low=-self._lim_up_acc, high=self._lim_up_acc, dtype=np.float64
        )

    def continuous_dynamics(self, x, t):  # pylint: disable=unused-argument
        vel = x[self._n : self._n * 2]
        xdot = np.concatenate((vel, self._action))
        return xdot



import numpy as np
from abc import abstractmethod
import time

from planarenvs.planar_common.planar_env import PlanarEnv


class MobileBaseEnv(PlanarEnv):

    BASE_HEIGHT = 1.0  # [m]
    BASE_WIDTH = 1.0  # [m]
    LINK_MASS_BASE = 500.0  #: [kg] mass of link 1

    MAX_VEL = 1
    MAX_POS = 5.0
    MAX_ACC = 1.0
    MAX_FOR = 100

    def __init__(self, render=False, dt=0.01):
        super().__init__(render=render, dt=dt)
        self.n = 1
        self._lim_up_pos = np.ones(self._n) * self.MAX_POS
        self._lim_up_vel = np.ones(self._n) * self.MAX_VEL
        self._lim_up_acc = np.ones(self._n) * self.MAX_ACC
        self._lim_up_for = np.ones(self._n) * self.MAX_FOR
        self.set_spaces()

    @abstractmethod
    def set_spaces(self):
        pass

    def _reward(self):
        reward = -1.0 if not self._terminal() else 0.0
        return reward

    def _terminal(self):
        if (
            self._state["joint_state"]["position"][0] > self.MAX_POS
            or self._state["joint_state"]["position"][0] < -self.MAX_POS
        ):
            return True
        return False

    @abstractmethod
    def continuous_dynamics(self, x, t):
        pass

    def render(self, mode="human"):
        bound = self.MAX_POS + 1.0
        bounds = [bound, bound]
        self.render_common(bounds)
        from gym.envs.classic_control import rendering #pylint: disable=import-outside-toplevel

        # drawAxis
        self._viewer.draw_line((-bound - 0.5, 0), (bound + 0.5, 0))

        p0 = [self._state["joint_state"]["position"][0], 0.5 * self.BASE_HEIGHT]
        tf = rendering.Transform(rotation=0, translation=p0)
        l, r, t, b = (
            -0.5 * self.BASE_WIDTH,
            0.5 * self.BASE_WIDTH,
            0.5 * self.BASE_HEIGHT,
            -0.5 * self.BASE_HEIGHT,
        )
        link = self._viewer.draw_polygon([(l, b), (l, t), (r, t), (r, b)])
        link.set_color(0, 0.8, 0.8)
        link.add_attr(tf)
        time.sleep(self.dt())

        return self._viewer.render(return_rgb_array=mode == "rgb_array")



import numpy as np
from gym import spaces

from planarenvs.ground_robots.envs.ground_robot_arm_env import GroundRobotArmEnv


class GroundRobotArmVelEnv(GroundRobotArmEnv):
    def set_spaces(self):
        self.observation_space = spaces.Dict(
                {"joint_state": spaces.Dict({
                    "position": spaces.Box(
                        low=np.concatenate((-self._lim_up_pos, -self._lim_up_arm_pos)),
                        high=np.concatenate((self._lim_up_pos, self._lim_up_arm_pos)),
                        dtype=np.float64
                ),
                    "velocity": spaces.Box(
                        low=np.concatenate((-self._lim_up_vel, -self._lim_up_arm_vel)),
                        high=np.concatenate((self._lim_up_vel, self._lim_up_arm_vel)),
                        dtype=np.float64
                ),
                "forward_velocity": spaces.Box(
                    low=-self._lim_up_rel_vel,
                    high=self._lim_up_rel_vel,
                    dtype=np.float64,
                ),
            })
            }
        )
        a = np.concatenate((self._lim_up_rel_vel, self._lim_up_arm_vel))
        self.action_space = spaces.Box(low=-a, high=a, dtype=np.float64)

    def integrate(self):
        super().integrate()
        qdot = self._action[2:]
        self._state["joint_state"]["forward_velocity"] = self._action[0:2]
        xdot_base = self.compute_xdot(
            self._state["joint_state"]["position"][0:3], self._state["joint_state"]["forward_velocity"]
        )
        self._state["joint_state"]["velocity"] = np.concatenate((xdot_base, qdot))

    def continuous_dynamics(self, x, t):
        # state = [x, y, theta, q, vel_rel, qdot]
        qdot = self._action[2:]
        x_base_pos = x[0:3]
        xdot_base = self.compute_xdot(x_base_pos, self._action[0:2])
        veldot = np.zeros(2)
        xddot = np.zeros(3)
        qddot = np.zeros(self._n_arm)
        return np.concatenate((xdot_base, veldot, xddot, qdot, qddot))



from gym.envs.registration import register

register(
    id="ground-robot-vel-v0",
    entry_point="planarenvs.ground_robots.envs:GroundRobotVelEnv",
)
register(
    id="ground-robot-acc-v0",
    entry_point="planarenvs.ground_robots.envs:GroundRobotAccEnv",
)
register(
    id="ground-robot-arm-acc-v0",
    entry_point="planarenvs.ground_robots.envs:GroundRobotArmAccEnv",
)
register(
    id="ground-robot-arm-vel-v0",
    entry_point="planarenvs.ground_robots.envs:GroundRobotArmVelEnv",
)



from planarenvs.sensors.sensor_common import Sensor
from planarenvs.sensors.sensor_common import dist2circ
import numpy as np


class GoalSensor(Sensor):
    def __init__(self, nb_goals=0, lim_sensor=10, mode="position"):
        super().__init__(nb_observations=nb_goals, lim_sensor=lim_sensor)
        self._observation = (
            np.ones([self._nb_observations, 2]) * self._lim_sensor
        )
        self._mode = mode
        self.name = mode

    @Sensor.name.setter
    def name(self, mode):
        if mode == "position":
            self._name = "GoalPosition"
        elif mode == "distance":
            self._name = "GoalDistance"

    def _reset(self):
        self._observation[:] = 0

    def sense(self, state, goals, obstacles, t=0):
        self._reset()
        if self._mode == "position":
            for idx, goal in enumerate(goals):
                if idx >= self._nb_observations:
                    break
                self._observation[idx] = goal.position(t=t)

        elif self._mode == "distance":
            for idx, goal in enumerate(goals):
                if idx >= self._nb_observations:
                    break
                goal_position = goal.position(t=t)
                goal_distance = dist2circ(
                    state["joint_state"]["position"], goal_position, goal.epsilon()
                )
                self._observation[idx] = goal_distance

        return self._observation.clip(-self._lim_sensor, self._lim_sensor)



from gym.envs.registration import register

register(
    id="nLink-reacher-acc-v0",
    entry_point="planarenvs.n_link_reacher.envs:NLinkAccReacherEnv",
)
register(
    id="nLink-reacher-vel-v0",
    entry_point="planarenvs.n_link_reacher.envs:NLinkVelReacherEnv",
)



from planarenvs.ground_robots.envs.vel import GroundRobotVelEnv
from planarenvs.ground_robots.envs.acc import GroundRobotAccEnv
from planarenvs.ground_robots.envs.acc_arm import GroundRobotArmAccEnv
from planarenvs.ground_robots.envs.vel_arm import GroundRobotArmVelEnv



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['planarenvs',
 'planarenvs.ground_robots',
 'planarenvs.ground_robots.envs',
 'planarenvs.mobile_base',
 'planarenvs.mobile_base.envs',
 'planarenvs.mobile_robot',
 'planarenvs.mobile_robot.envs',
 'planarenvs.n_link_reacher',
 'planarenvs.n_link_reacher.envs',
 'planarenvs.n_link_reacher.resources',
 'planarenvs.planar_common',
 'planarenvs.point_robot',
 'planarenvs.point_robot.envs',
 'planarenvs.scenes',
 'planarenvs.sensors']

package_data = \
{'': ['*']}

install_requires = \
['forwardkinematics>=1.0.3,<2.0.0',
 'gym>=0.22,<0.23',
 'mpscenes>=0.1,<0.2',
 'numpy>=1.19,<1.24',
 'pyglet>=1.5.21,<2.0.0',
 'pylint>=2.13.3,<3.0.0',
 'scipy>=1.5,<2.0']

setup_kwargs = {
    'name': 'planarenvs',
    'version': '1.2.1',
    'description': 'Lightweight open-ai gym environments for planar kinematic chains.',
    'long_description': 'None',
    'author': 'Max Spahn',
    'author_email': 'm.spahn@tudelft.nl',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)



import numpy as np
from numpy import pi
import time
from abc import abstractmethod

from planarenvs.planar_common.planar_env import PlanarEnv
from forwardkinematics.planarFks.planarArmFk import PlanarArmFk


class NLinkReacherEnv(PlanarEnv):

    LINK_LENGTH = 1.0  # [m]
    LINK_MASS = 1.0

    MAX_VEL = 4 * pi
    MAX_POS = pi
    MAX_ACC = 9 * pi
    MAX_TOR = 1000

    def __init__(self, render=False, n=2, dt=0.01):
        super().__init__(render=render, dt=dt)
        self.n = n
        self._lim_up_pos = np.ones(self._n) * self.MAX_POS
        self._lim_up_vel = np.ones(self._n) * self.MAX_VEL
        self._lim_up_acc = np.ones(self._n) * self.MAX_ACC
        self._lim_up_tor = np.ones(self._n) * self.MAX_TOR
        self.set_spaces()
        self._fk = PlanarArmFk(self._n)

    @abstractmethod
    def set_spaces(self):
        pass

    def _terminal(self):
        return False

    def _reward(self):
        reward = -1.0 if not self._terminal() else 0.0
        return reward

    @abstractmethod
    def continuous_dynamics(self, x, t):
        pass

    def render(self, mode="human"):
        bound = self.LINK_LENGTH * self._n + 0.2
        bounds = [bound, bound]
        self.render_common(bounds)

        # axis
        self._viewer.draw_line((-bound, 0), (bound, 0))
        self.render_base()
        for i in range(self._n):
            self.render_link(i)
        self.render_end_effector()
        time.sleep(self.dt())
        return self._viewer.render(return_rgb_array=mode == "rgb_array")

    def render_base(self):
        from gym.envs.classic_control import rendering #pylint: disable=import-outside-toplevel

        base = self._viewer.draw_polygon(
            [(-0.2, 0), (0.0, 0.2), (0.2, 0), (-0.2, 0)]
        )
        tf0 = rendering.Transform(rotation=0, translation=(0.0, -0.2))
        base.add_attr(tf0)

    def render_link(self, i):
        from gym.envs.classic_control import rendering #pylint: disable=import-outside-toplevel

        l, r, t, b = 0, self.LINK_LENGTH, 0.01, -0.01
        fk = self._fk.fk(self._state["joint_state"]["position"], i)
        tf = rendering.Transform(rotation=fk[2], translation=fk[0:2])
        link = self._viewer.draw_polygon([(l, b), (l, t), (r, t), (r, b)])
        link.set_color(0, 0.8, 0.8)
        link.add_attr(tf)
        joint = self._viewer.draw_circle(0.10)
        joint.set_color(0.8, 0.8, 0)
        joint.add_attr(tf)

    def render_end_effector(self):
        from gym.envs.classic_control import rendering #pylint: disable=import-outside-toplevel

        fk = self._fk.fk(self._state["joint_state"]["position"], self._n)
        tf = rendering.Transform(rotation=fk[2], translation=fk[0:2])
        eejoint = self._viewer.draw_circle(0.10)
        eejoint.set_color(0.8, 0.8, 0)
        eejoint.add_attr(tf)
